#!/usr/bin/env python
#
# Copyright (c) 2008  Simon Peter. All rights reserved.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.  You should have
# received a copy of the GNU General Public License along with this program; if
# not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
# Floor, Boston, MA 02110-1301, USA. 
#
# Author(s): Simon Peter <probono@myrealbox.com>

import os
import sys
import dbus
import subprocess
import shutil
import tempfile

bus = dbus.SystemBus () 
hal_obj = bus.get_object ('org.freedesktop.Hal', '/org/freedesktop/Hal/Manager')
hal = dbus.Interface (hal_obj, 'org.freedesktop.Hal.Manager')

class USBDevice(object):
    """By making a class out of an USB stick, we can display a "nice name" 
    but have all the details handy. 
    Unfortunately the info returned for the device mounted to /cdrom is not complete 
    (smells buggy). Maybe using UDIs instead of device names such as "/dev/sda" would solve this.
    It should contain no GUI-specific code for reusability. 
    """
    def __init__(self,device):
        self.device = device # such as /dev/sdb
        self.device1 = self.device + "1" # such as /dev/sdb1
        self.device2 = self.device + "2" # such as /dev/sdb2 # for home-rw
        self.udi = hal.FindDeviceStringMatch('block.device',self.device)[0] 
        self.obj = bus.get_object("org.freedesktop.Hal", self.udi)
        self.iface = dbus.Interface(self.obj, 'org.freedesktop.Hal.Device')
        self.vendor = self.getInfo("vendor")
        self.product = self.getInfo("product")
        self.partitions = self.listPartitions()
        self.mountpoints = self.listMountpoints()
        self.size = self.getSizeMB()

    def __repr__(self):
        """Return a nice human readable string"""
        return self.vendor + " " + self.product + " (" + self.device + ") " + str(self.size) + " MB"

    def getSizeMB(self):
        try:
            result = self.iface.GetProperty("storage.removable.media_size")/1024/1024 # in MB
            return result
        except:
            return 0

    def getInfo(self,requestedinfo):
        try:
            result = self.iface.GetProperty("info." + requestedinfo)
        except:
            result = "unknown " + requestedinfo
        if result == None:
            result = "unknown " + requestedinfo
        return result

    def listPartitions(self):
        """Returns a list of all device labels 
        of this USB storage device
        """
        partitions = []
        childudis = hal.FindDeviceStringMatch("info.parent",self.udi)
        for childudi in childudis:
            childobj = bus.get_object("org.freedesktop.Hal", childudi)
            childiface = dbus.Interface(childobj, 'org.freedesktop.Hal.Device')
            try:
                partitions.append(childiface.GetProperty("volume.label"))
            except:
                partitions.append("unknownpartition")
        return partitions

    def listMountpoints(self):
        """Returns a list of all mountpoints that 
        partitions of this USB storage device are currently mounted at
        """
        mountpoints = []
        childudis = hal.FindDeviceStringMatch("info.parent",self.udi)
        for childudi in childudis:
            childobj = bus.get_object("org.freedesktop.Hal", childudi)
            childiface = dbus.Interface(childobj, 'org.freedesktop.Hal.Device')
            try:
                mountpoints.append(childiface.GetProperty("volume.mount_point"))
            except:
                mountpoints.append("unknown")
        return mountpoints

    def unmountAllPartitions(self):
        """Unmounts all mounted partitions of this device."""
        mountpoints = self.listMountpoints()
        print mountpoints
        for mountpoint in mountpoints:
            if mountpoint == "":
                pass
            else:
                try:
                    returncode = subprocess.call(["umount", mountpoint])
                    if returncode != 0:
                        return False # if the process returned an error return code
                except:
                    return False
        return True
    
    def runsfdisk(self, sfdiskcmds):
        """Runs sfdisk with the commands supplied."""
        proc = subprocess.Popen(shell = False,
                                stdin = subprocess.PIPE,
                                stdout = subprocess.PIPE,
                                stderr = subprocess.PIPE,
                                args = [ '/sbin/sfdisk', '-uM', self.device ])
        try:
            proc.stdin.write(sfdiskcmds)
            (output, error) = proc.communicate()
            proc.wait()
        except:
            return False # if we couldn't run sfdisk at all
        if proc.returncode != 0:
            return False # if running sfdisk returned an error
        else:
            return True

    def makeOneBootableVFAT32Partition(self):
        """Creates one bootable partition on the device to be formatted with VFAT32."""
        result = self.runsfdisk(",,06,*\n")
        if result == True:
            return True
        else:
            return False

    def makeTwoPartitionsForPersistence(self):
        """Creates one bootable partition on the device to be formatted with VFAT32."""
        result = self.runsfdisk(",800,06,*\n,,83,-\n")
        if result == True:
            return True
        else:
            return False

    def formatFirstPartitionVFAT32(self):
        """Formats the first partition with VFAT32."""
        try:
            command = ["/sbin/mkfs.vfat", "-F", "32", "-n", "live", 
self.device1]
            returncode = subprocess.call(command) # call waits for the process to complete
        except:
            return False # if we could not run the command at all
        if returncode != 0:
            return False # if the process returned an error return code
        return True

    def formatSecondPartitionExt2(self):
        """Formats the second partition with ext2."""
        try:
            command = ["mkfs.ext2", "-b", "4096", "-L" "home-rw", self.device2]
            returncode = subprocess.call(command) # call waits for the process to complete
        except:
            return False # if we could not run the command at all
        if returncode != 0:
            return False # if the process returned an error return code
        return True
    
    def writeMBR(self):
        """Writes MBR to the device"""
        try:
            mbr = file("/usr/lib/syslinux/mbr.bin","r")
            device = file(self.device,"w")
            shutil.copyfileobj(mbr,device)
            device.close()
            mbr.close()
        except:
            return False
        return True

    def mountFirstPartition(self):
        try:
            tempmountpoint = tempfile.mkdtemp("","tmplive","/mnt") # make temp mountpoint in /mnt
            command = ["mount", "-o", "sync", self.device1, tempmountpoint]
            subprocess.call(command)
            self.mountpoints = self.listMountpoints() # refresh list of mountpoints
        except:
            return False
        return True

if __name__ == "__main__":
    "Testing if module is directly called"
    myfoo = USBDevice("/dev/sdb")
    print myfoo
    print myfoo.product
    print myfoo.partitions
    print myfoo.mountpoints

